<?php

require_once MAX_PATH . '/lib/max/Delivery/common.php';

/**
 * A class for testing the common.php functions.
 *
 * @package    MaxDelivery
 * @subpackage TestSuite
 * @author
 *
 */
class test_DeliveryCommon extends UnitTestCase
{

    /**
     * The constructor method.
     */
    function test_DeliveryCommon()
    {
        $this->UnitTestCase();
    }


	/**
	 * Test1: secure URL
	 * Test2: unsecure URL
	 * A function that can be used to get the delivery URL,
	 * or the delivery URL prefix (sans-file) if no filname
	 * is passed in.
	 *
	 * @param string $file Optional delivery file name.
	 * @return string The delivery URL.
	 */
	function test_MAX_commonGetDeliveryUrl()
	{
		$this->sendMessage('test_MAX_commonGetDeliveryUrl');
        $GLOBALS['_MAX']['CONF']['webpath']['delivery'] 	= 'www.maxstore.net/www/delivery';
        $GLOBALS['_MAX']['CONF']['webpath']['deliverySSL'] 	= 'secure.maxstore.net/www/delivery';

		$file 	= 'test.html';

		// Test1
        $GLOBALS['_MAX']['CONF']['max']['sslPort']	= $_SERVER['SERVER_PORT'];
 		$ret 	= MAX_commonGetDeliveryUrl($file);
        $this->assertEqual($ret, 'https://secure.maxstore.net:'.$_SERVER['SERVER_PORT'].'/www:'.$_SERVER['SERVER_PORT'].'/delivery/test.html');
        // Test2
        $GLOBALS['_MAX']['CONF']['max']['sslPort']	= null;
		$ret 	= MAX_commonGetDeliveryUrl($file);
        $this->assertEqual($ret, 'http://www.maxstore.net/www/delivery/test.html');
	}

	/**
	 * Test1 return a URL
	 * A function to generate the URL for delivery scripts.
	 *
	 * @param string $file The file name of the delivery script.
	 * @return string The URL to the delivery script.
	 */
	function test_MAX_commonConstructDeliveryUrl()
	{
		$this->sendMessage('test_MAX_commonConstructDeliveryUrl');
        $GLOBALS['_MAX']['CONF']['webpath']['delivery'] 	= 'www.maxstore.net/www/delivery';
		$file 	= 'test.html';
		$ret 	= MAX_commonConstructDeliveryUrl($file);
        $this->assertEqual($ret, 'http://www.maxstore.net/www/delivery/test.html');
	}

	/**
	 * Test1 return secure URL
	 * A function to generate the secure URL for delivery scripts.
	 *
	 * @param string $file The file name of the delivery script.
	 * @return string The URL to the delivery script.
	 */
	function test_MAX_commonConstructSecureDeliveryUrl()
	{
		$this->sendMessage('test_MAX_commonConstructSecureDeliveryUrl');
        $GLOBALS['_MAX']['CONF']['webpath']['deliverySSL'] 	= 'secure.maxstore.net/www/delivery';
        $GLOBALS['_MAX']['CONF']['max']['sslPort'] 			= 444;
		$file = 'test.html';
		$ret 	= MAX_commonConstructSecureDeliveryUrl($file);
        $this->assertEqual($ret, 'https://secure.maxstore.net:444/www:444/delivery/test.html');
	}

	/**
	 * Test1: return unsecure URL
	 * Test2: return secure URL
	 * A function to generate the URL for delivery scripts without a protocol.
	 *
	 * @param string $file The file name of the delivery script.
	 * @param boolean $ssl Use the SSL delivery path (true) or not. Default is false.
	 * @return string The parital URL to the delivery script (i.e. without
	 *                an 'http:' or 'https:' prefix).
	 */
	function test_MAX_commonConstructPartialDeliveryUrl()
	{
		$this->sendMessage('test_MAX_commonConstructPartialDeliveryUrl');
        $GLOBALS['_MAX']['CONF']['webpath']['deliverySSL'] 	= 'secure.maxstore.net/www/delivery';
        $GLOBALS['_MAX']['CONF']['webpath']['delivery'] 	= 'www.maxstore.net/www/delivery';
		$file = 'test.html';
		// Test1
		$ret = MAX_commonConstructPartialDeliveryUrl($file, false);
        $this->assertEqual($ret, '//www.maxstore.net/www/delivery/test.html');
        // Test2
		$ret = MAX_commonConstructPartialDeliveryUrl($file, true);
        $this->assertEqual($ret, '//secure.maxstore.net/www/delivery/test.html');
	}

	/**
	 * Remove an assortment of special characters from a variable or array:
	 * 1.  Strip slashes if magic quotes are turned on.
	 * 2.  Strip out any HTML
	 * 3.  Strip out any CRLF
	 * 4.  Remove any white space
	 *
	 * @access  public
	 * @param   string $var  The variable to process.
	 * @return  string       $var, minus any special quotes.
	 */
	function test_MAX_commonRemoveSpecialChars()
	{
		$this->sendMessage('test_MAX_commonRemoveSpecialChars');
		if (get_magic_quotes_gpc())
		{
			$strIn0	= "Mr O\'Reilly<br />".chr(13).chr(10);
			$strIn1	= "\'Mr Reilly\'\r\n";
		}
		else {
			$strIn0	= "Mr O'Reilly<br />".chr(13).chr(10);
			$strIn1	= "'Mr Reilly'\r\n";
		}

		$strRe0 = "Mr O'Reilly";
		$strRe1 = "'Mr Reilly'";
		$aIn	= array(0 => $strIn0,
						1 => array(0 => $strIn1),
						);
		MAX_commonRemoveSpecialChars($aIn);
		$prn    = var_export($aIn[1][0], true);

        $this->assertEqual($aIn[0], $strRe0);
        $this->assertEqual($aIn[1][0], $strRe1,'test_MAX_commonRemoveSpecialChars');
	}

	/**
	 * TODO: implement as web test
	 * This function sends the anti-caching headers when called
	 *
	 */
	function test_MAX_commonSetNoCacheHeaders()
	{
		$this->sendMessage('test_MAX_commonSetNoCacheHeaders: Not Implemented');
	}

	/**
	 *
	 * This function takes an array of variable names
	 * and makes them available in the global scope
	 *
	 * $_POST values take precedence over $_GET values
	 *
	 */
	function test_MAX_commonRegisterGlobals()
	{
	    $tmpGlobals = $GLOBALS;
	    $_GET['max_test_get']      = '0';
	    $_POST['max_test_get']     = '1';
	    $_GET['max_test_post']     = '0';
	    $_POST['max_test_post']    = '1';
		MAX_commonRegisterGlobals('max_test_get', 'max_test_post');
		$this->assertTrue(array_key_exists('max_test_get', $GLOBALS),'max_test_get exists');
		$this->assertTrue(array_key_exists('max_test_post', $GLOBALS),'max_test_post exists');
		$this->assertTrue($GLOBALS['max_test_get'],'GLOBALS precedence error');
		$this->assertTrue($GLOBALS['max_test_post'],'GLOBALS precedence error');
		$GLOBALS = $tmpGlobals;
	}

	/**
	 * Recursivley add slashes to the values in an array
	 *
	 * @param array Input array
	 * @return array Output array with values slashed
	 */
	function test_MAX_commonSlashArray()
	{
		$this->sendMessage('test_MAX_commonSlashArray');
		$strIn0	= "Mr O\'Reilly";
		$strIn1	= '"Mr Reilly"\n';
		$strRe0 = addslashes($strIn0);
		$strRe1 = addslashes($strIn1);

		$aIn	= array(0 => $strIn0,
						1 => array(0 => $strIn1),
						);
		$aRet 	= MAX_commonSlashArray($aIn);

        $this->assertEqual($aRet[0], $strRe0);
        $this->assertEqual($aRet[1][0], $strRe1);
	}

	/**
	 * Test1: return urldecoded/decrypted string
	 * Test2: return urldecoded string
	 *
	 * This function takes the "source" value and normalises it
	 * and encrypts it if necessary
	 *
	 * @param string The value from the source parameter
	 * @return string Encrypted source
	 */
	function test_MAX_commonDeriveSource()
	{
        $str = "http://www.somewhere.com/index.php?a=this is a file&b=43589uhhjkh";
	    // Test1: obfuscate (encrypt) and return
	    $source = urlencode($str);
		$GLOBALS['_MAX']['CONF']['delivery']['obfuscate'] = 1;
		$expect = '{obfs:220208208212266277277205205205278209213215223205220223210223278225213215277219214224223204278212220212261227263208220219209292219209292227292222219216223286226263272273271268267207220220218217220}';
        $return = MAX_commonDeriveSource($source);
        $this->assertEqual($return, $expect);
        // Test2: urldecode and return
	    $source = urlencode($str);
		$GLOBALS['_MAX']['CONF']['delivery']['obfuscate'] = 0;
		$expect = $str;
        $return = MAX_commonDeriveSource($source);
        $this->assertEqual($return, $expect);
	}

    /**
     * Test the _MAX_commonEncrypt and _MAX_commonDecrypt functions.
     */
	function test_MAX_commonEncryptDecrypt()
	{
		$this->sendMessage('test_MAX_commonEncryptDecrypt');
		$string = 'Lorem ipsum dolor sit amet ipso facto';
		$GLOBALS['_MAX']['CONF']['delivery']['obfuscate'] = 1;
		$retEnc = MAX_commonEncrypt($string);
        $this->assertNotEqual($retEnc, $string);
		$retDec = MAX_commonDecrypt($retEnc);
        $this->assertNotEqual($retDec, $retEnc);
        $this->assertEqual($retDec, $string);

		$GLOBALS['_MAX']['CONF']['delivery']['obfuscate'] = 0;
		$retDec = MAX_commonDecrypt($string);
        $this->assertEqual($retDec, $string);
	}

    function test_MAX_commonInitVariables()
    {
		$GLOBALS['_MAX']['CONF']['delivery']['obfuscate'] = 1;
        $return = MAX_commonInitVariables();
        global $context, $source, $target, $withText, $withtext, $ct0, $what, $loc, $referer, $zoneid, $campaignid, $bannerid;
        $this->assertEqual($context, array(), '$context');
        $this->assertEqual($source, '{obfs:}', '$source'); // only if conf->obfuscate
        $this->assertEqual($target, '_blank', '$target');
        $this->assertEqual($withText, '', '$withText');
        $this->assertEqual($withtext, '', '$withtext');
        $this->assertEqual($withtext, $withText, '$withtext/$withText');
        $this->assertEqual($ct0, '', '$ct0');
        $this->assertEqual($what, '', '$what');
        $this->assertTrue(in_array($loc, array(stripslashes($loc),$_SERVER['HTTP_REFERER'],'')));
        $this->assertEqual($referer, null, '$referer');
        $this->assertEqual($zoneid, null, '$zoneid');
        $this->assertEqual($campaignid, null, '$campaignid');
        $this->assertEqual($bannerid, null, '$bannerid');
    }

    /**
     * @todo create this as web test as it sends headers
     *
     */
    function test_MAX_commonDisplay1x1()
    {
        $this->assertTrue(true);
    }


}

?>
