<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id$
*/

//require_once MAX_PATH . '/lib/max/core/Task/Runner.php';
require_once MAX_PATH . '/lib/max/DB.php';
require_once MAX_PATH . '/lib/max/SqlBuilder.php';
require_once MAX_PATH . '/lib/max/Table/Core.php';
require_once MAX_PATH . '/lib/max/core/ServiceLocator.php';
require_once MAX_PATH . '/lib/max/Delivery/common.php';
require_once MAX_PATH . '/lib/max/Delivery/adSelect.php';
require_once MAX_PATH . '/lib/max/Maintenance/Priority/AdServer.php';
require_once MAX_PATH . '/lib/max/Dal/Maintenance/Priority.php';

/**
 * A class for simulating maintenance/delivery scenarios
 *
 * @package
 * @subpackage
 * @author
 */
class SimulationScenario
{
    var $sourceFile = '';
    var $sourceData = '';

    var $requestFile = '';
    var $requestData = '';

    var $totalDelivery = 0;
    var $totalRequests = 0;

    var $aDelivered = array();
    var $aFailed = array();

    //var $aTasks = array();

    var $scenarioConfig = '';

    //var $oTaskRunner;
    var $oServiceLocator;
    var $oCoreTables;
    var $oDBH;

    /**
     * The constructor method.
     */
    function SimulationScenario()
    {
    }

    /**
     * initialisation
     *
     * load the 'requestset' config file
     * do not load the dataset yet - allow child scenario to choose
     *
     * @param string $filename - name of scenario's dataset and config
     * @param string $dbname database name
     */
    public function init($filename, $dbname = 'max03_sim')
    {
        $this->sourceFile = 'data/'.$filename.'.sql';
        $this->requestFile = 'data/'.$filename.'.php';
        $this->loadRequestset();

        // Create the task runner object, for running the MPE tasks
        //$this->oTaskRunner = new MAX_Core_Task_Runner();

        $GLOBALS['_MAX']['CONF']['database']['name'] = $dbname;

        MAX_Table_Core::destroy();
        $this->oCoreTables = MAX_Table_Core::singleton();

        $this->oServiceLocator = &ServiceLocator::instance();
        $this->oServiceLocator->register('MAX_TABLES', $tables);

        // MAX_TABLES instance registers MAX_DB
        $this->oDBH = &$this->oServiceLocator->get('MAX_DB');

        $this->setDateTime();
    }

    /**
     * run the list of tasks
     *
     * @todo tasks
     */
    public function run()
    {
        //$this->oTaskRunner->runTasks();
    }

    /**
     * add a task to the task list
     *
     * @todo add task
     *
     * @param object $obj
     */
    public function addTask(&$obj)
    {
        //$this->oTaskRunner->addTask($obj);
    }

    /**
     * create a new set of max tables
     *
     */
    public function newTables()
    {
        if ($this->oDBH)
        {
            $this->oCoreTables->createAllTables();
        }
        else
        {
            $this->reportResult(false, 'could not create new tables, invalid db object', $this->oDBH);
        }
    }

    /**
     * execute a set of ad requests
     *
     * @param int $iteration : the id of the current iteration (*hour*)
     */
    public function makeRequests($iteration)
    {
        $aIteration = $this->scenarioConfig['aIterations'][$iteration];
        $requestObjs = count($aIteration['request_objects']);
        $k = 0;
        for($i=1;$i<=$aIteration['max_requests'];$i++)
        {
            if ($aIteration['shuffle_requests'])
            {
                $k = rand(1, $requestObjs);
            }
            else
            {
                $k = ($k==$requestObjs ? 1 : $k+1 );
            }
            $result = $this->_makeRequest($aIteration['request_objects'][$k]);
            $this->_recordDelivery($iteration, $result['bannerid']);
        }
       $this->_summariseInterval($iteration, $i-1);
       $this->_incDateTime();
    }

    /**
     * increment the success or failure values in arrays
     * increment the totalDelivery value
     * increment the totalRequests value
     *
     * @param int $iteration
     * @param int $bannerId - if null then delivery failed
     */
    private function _recordDelivery($iteration, $bannerId)
    {
        //create the delivered success/failure array keys for current iteration
		if (! array_key_exists($iteration,$this->aDelivered))
		{
		    $this->aDelivered[$iteration] = array();
		}
		if (! array_key_exists($iteration,$this->aFailed))
		{
    		$this->aFailed[$iteration]  = array();
		}
        if ($bannerId)
        {
            $array = &$this->aDelivered;
            $this->totalDelivery++;
        }
        else
        {
            $array = &$this->aFailed;
    		$bannerId = 0;
        }
	    if (!array_key_exists($bannerId,$array[$iteration]))
	    {
	        $array[$iteration][$bannerId] = 0;
	    }
        $array[$iteration][$bannerId]++;
        $this->totalRequests++;
    }

    /**
     * make a request via adSelect function
     *
     * @param string $what
     * @return array of ad info
     */
    private function _makeRequest($oRequest)
    {
        return MAX_adSelect(
                            $oRequest->what,
                            $oRequest->target,
                            $oRequest->source,
                            $oRequest->withText,
                            $oRequest->context,
                            $oRequest->richMedia,
                            $oRequest->ct0,
                            $oRequest->loc,
                            $oRequest->referer
                           );
    }

    /**
     *
     * add up the delivery stats for this interval
     * store to data_summary_ad_zone table
     *
     * @param int $hour
     * @param int $requests
     */
    private function _summariseInterval($interval, $requests)
    {
       $this->printMessage('summary hourly insert: '. $interval);
       $date = $this->_getDateTimeString();
       if (count($this->aFailed[$interval])>0)
       {
            $this->printMessage('failures: '.$this->aFailed[$interval][0]);
       }
       if (count($this->aDelivered[$interval])==0)
       {
            $this->printMessage('nothing delivered');
            return ;
       }
       foreach($this->aDelivered[$interval] as $bannerid => $impressions)
       {
           // need to get/put zone id?
            $aValues = array(
                              'day' => $date,
                              'hour' => $interval,
                              'ad_id' => $bannerid,
                              //'creative_id' => ,
                              //'zone_id' => ,
                              'requests' => $requests,
                              'impressions' => $impressions,
                              //'clicks' => ,
                              //'conversions' => ,
                              //'total_basket_value' => ,
                              //'total_num_items' => ,
                              //'total_revenue' => ,
                              //'total_cost' => ,
                              //'updated'
                            );
            $table = $GLOBALS['_MAX']['CONF']['table']['data_summary_ad_hourly'];
            $aTable = array($table=>$table);
            $result = SqlBuilder::_insert($aTable, $aValues);
            $this->dumpVar($aValues);
            if (!$result)
            {
                $this->reportResult($result, 'summary hourly insert', $aValues);
                exit();
            }
       }
    }

    public function runPriority()
    {
        $this->printMessage('updatePriorities: started');
        $oMaintenancePriority = new MAX_Maintenance_Priority_AdServer();
        $oMaintenancePriority->updatePriorities();
        $this->printMessage('updatePriorities: finished');
    }

    /**
     * tweak the date
     *
     * @param int $hour
     * @param string $day
     */
    public function setDateTime($hour=0, $day='2000-01-01')
    {
        $date.= $day.' '.str_pad($hour, 2, '0', STR_PAD_LEFT).':00:00';
        $this->oServiceLocator->register('now', new Date($date));
    }

    private function _getDateTimeString()
    {
        $oDate = $this->oServiceLocator->get('now');
        if (!is_a($oDate,'Date'))
        {
            $this->setDateTime();
            $oDate = $this->oServiceLocator->get('now');
        }
        return $oDate->getDate();
    }

    private function _incDateTime()
    {
        $oSpan = new Date_Span();
        $oSpan->hour = 1;
        $oDate = $this->oServiceLocator->get('now');
        $oDate->addSpan($oSpan);
        $this->oServiceLocator->register('now', $oDate);
    }

    /**
     * read the scenario config file
     *
     */
    public function loadRequestset()
    {
        if (file_exists($this->requestFile))
        {
            require_once($this->requestFile);
            $this->scenarioConfig = $GLOBALS['_MAX']['CONF']['sim'];
        }
        else
        {
            $this->reportResult(false, 'exists file', $this->requestFile);
            exit();
        }
    }

    /**
     * read a .sql file
     * parse into individual queries
     * execute the queries
     *
     * @todo insert table prefix!!!
     */
    public function loadDataset()
    {
        $patternDrop     = '/(DROP TABLE IF EXISTS [\w\W\s]+;)/U';
        $patternCreate   = '/(CREATE TABLE IF NOT EXISTS [\w\W\s]+;)/U';
        $patternInsert   = '/(INSERT INTO [\w\W\s]+\);)/U';
        $patternTruncate = '/(TRUNCATE TABLE [\W\w\s]+;)/U';

        $prefix = $GLOBALS['_MAX']['CONF']['database']['prefix'];

        if (file_exists($this->sourceFile))
        {
            $this->sourceData = file_get_contents($this->sourceFile);

            if ($this->sourceData)
            {
                $i	= preg_match_all($patternDrop, $this->sourceData, $aDrop);
        		if ($i)
        		{
                    foreach ($aDrop[0] as $k => $v)
                    {
                        $v = preg_replace('/TABLE[\s]+`([\w]+)`/U', 'TABLE `'.$prefix.'$1`',$v);
                        $query = $v;
                        $result = $this->oDBH->query($query);
                        if (!$result)
                        {
                            $this->reportResult($result, 'dropped tables', $query);
                            exit();
                        }
                    }
        		}
        		$i	= preg_match_all($patternCreate, $this->sourceData, $aCreate);
        		if ($i)
        		{
                    foreach ($aCreate[0] as $k => $v)
                    {
                        $v = preg_replace('/TABLE[\s]+`([\w]+)`/U', 'TABLE `'.$prefix.'$1`',$v);
                        $query = $v;
                        $result = $this->oDBH->query($query);
                        if (!$result)
                        {
                            $this->reportResult($result, 'created tables', $query);
                            exit();
                        }
                    }
        		}
                $i	= preg_match_all($patternTruncate, $this->sourceData, $aTruncate);
        		if ($i)
        		{
                    foreach ($aTruncate[0] as $k => $v)
                    {
                        $v = preg_replace('/TABLE[\s]+`([\w]+)`/U', 'TABLE `'.$prefix.'$1`',$v);
                        $query = $v;
                        $result = $this->oDBH->query($query);
                        if (!$result)
                        {
                            $this->reportResult($result,  'truncated tables',$query);
                            exit();
                        }
                    }
        		}
                $i	= preg_match_all($patternInsert, $this->sourceData, $aInsert);
        		if ($i)
        		{
                    foreach ($aInsert[0] as $k => $v)
                    {
                        $v = preg_replace('/INSERT INTO[\s]+`([\w]+)`/U', 'INSERT INTO `'.$prefix.'$1`',$v);
                        $query = $v;
                        $result = $this->oDBH->query($query);
                        if (!$result)
                        {
                            $this->reportResult($result,  'inserted data',$query);
                            exit();
                        }
                    }
        		}
            }
            else
            {
                $this->reportResult(false, 'reading file', $this->sourceFile);
                exit();
            }
        }
        else
        {
            $this->reportResult(false, 'opening file', $this->sourceFile);
            exit();
        }
    }

    public function reportResult($result, $msg='', $var)
    {
        if ($msg)
        {
            if ($result)
            {
                $this->printMessage('success: '.$msg);
            }
            else
            {
                $this->printError('failed: '.$msg);
            }
        }
        $this->dumpVar($var);
    }

    private function dumpVar($var)
    {
        echo '<p>';
        print_r($var);
        echo '</p>';
    }

    public function printMessage($msg='')
    {
        if ($msg)
        {
           echo '<p>'.$msg.'</p>';
        }
    }
    public function printError($msg='')
    {
        if ($msg)
        {
           echo '<p style="background-colour:pink">'.$msg.'</p>';
        }
    }


}
?>