<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: TestRunner.php 4346 2006-03-06 16:43:19Z andrew@m3.net $
*/

require_once MAX_PATH . '/tests/testClasses/TestFiles.php';
require_once MAX_PATH . '/tests/testClasses/TestEnv.php';
require_once MAX_PATH . '/lib/simpletest/unit_tester.php';
require_once MAX_PATH . '/lib/simpletest/mock_objects.php';
require_once MAX_PATH . '/lib/simpletest/reporter.php';
require_once MAX_PATH . '/lib/simpletest/web_tester.php';

/**
 * A class for running tests.
 *
 * @package    Max
 * @subpackage TestSuite
 * @author     Andrew Hill <andrew@m3.net>
 */
class TestRunner
{

    /**
     * A method to run all the tests in the Max project.
     *
     * @static
     */
    function runAll()
    {
        $type = $GLOBALS['_MAX']['TEST']['test_type'];
        foreach ($GLOBALS['_MAX']['TEST'][$type . '_layers'] as $layer => $data) {
            // Run each layer test in turn
            TestRunner::runLayer($layer);
        }
    }

    /**
     * A method to run all tests in a layer.
     *
     * @static
     * @param string $layer The layer group to run.
     */
    function runLayer($layer)
    {
        $type = $GLOBALS['_MAX']['TEST']['test_type'];
        // Set up the environment for the test
        TestRunner::setupEnv($layer);
        // Find all the tests in the layer
        $tests = TestFiles::getLayerTestFiles($type, $layer);
        // Add the test files to a SimpleTest group
        $testName = strtoupper($type) . ': ' .
            $GLOBALS['_MAX']['TEST'][$type . '_layers'][$layer][0] .' Tests';
        $test = new GroupTest($testName);
        foreach ($tests as $layerCode => $folders) {
            foreach ($folders as $folder => $files) {
                foreach ($files as $index => $file) {
                    $test->addTestFile(MAX_PROJECT_PATH . '/' . $folder . '/' .
                                       constant($type . '_TEST_STORE') . '/' . $file);
                }
            }
        }
        $test->run(new HtmlReporter());
        // Tear down the environment for the test
        TestRunner::teardownEnv($layer);
    }

    /**
     * A method to all tests in a layer/folder.
     *
     * @static
     * @param string $layer  The layer group to run.
     * @param string $folder The folder group to run.
     */
    function runFolder($layer, $folder)
    {
        $type = $GLOBALS['_MAX']['TEST']['test_type'];
        // Set up the environment for the test
        TestRunner::setupEnv($layer);
        // Find all the tests in the layer/folder
        $tests = TestFiles::getTestFiles($type, $layer, MAX_PROJECT_PATH . '/' . $folder);
        // Add the test files to a SimpleTest group
        $testName = strtoupper($type) . ': ' .
            $GLOBALS['_MAX']['TEST'][$type . '_layers'][$layer][0] . ': Tests in ' . $folder;
        $test = new GroupTest($testName);
        foreach ($tests as $folder => $data) {
            foreach ($data as $index => $file) {
                $test->addTestFile(MAX_PROJECT_PATH . '/' . $folder . '/' .
                                   constant($type . '_TEST_STORE') . '/' . $file);
            }
        }
        $test->run(new HtmlReporter());
        // Tear down the environment for the test
        TestRunner::teardownEnv($layer);
    }

    /** A method to run a sinlge test file.
     *
     * @static
     * @param string $layer  The layer group to run.
     * @param string $folder The folder group to run.
     * @param string $file   The file to run.
     */
    function runFile($layer, $folder, $file)
    {
        $type = $GLOBALS['_MAX']['TEST']['test_type'];
        // Set up the environment for the test
        TestRunner::setupEnv($layer);
        // Add the test file to a SimpleTest group
        $testName = strtoupper($type) . ': ' .
            $GLOBALS['_MAX']['TEST'][$type . '_layers'][$layer][0] . ': ' . $folder . '/' . $file;
        $test = new GroupTest($testName);
        $test->addTestFile(MAX_PROJECT_PATH . '/' . $folder . '/' .
                           constant($type . '_TEST_STORE') . '/' . $file);
        $test->run(new HtmlReporter());
        // Tear down the environment for the test
        TestRunner::teardownEnv($layer);
    }

    /**
     * A method to set up the environment based on
     * the layer the test/s is/are in.
     *
     * @param string $layer The layer the test/s is/are in.
     */
    function setupEnv($layer)
    {
        $type = $GLOBALS['_MAX']['TEST']['test_type'];
        $envType = $GLOBALS['_MAX']['TEST'][$type . '_layers'][$layer][1];
        // Ensure the config file is fresh
        TestEnv::restoreConfig();
        // Setup the database, if needed
        if ($envType == DB_NO_TABLES) {
            TestEnv::setupDB();
        } elseif ($envType == DB_WITH_TABLES) {
            TestEnv::setupDB();
            TestEnv::setupCoreTables();
        } elseif ($envType == DB_WITH_DATA) {
            TestEnv::setupDB();
            TestEnv::setupCoreTables();
            TestEnv::setupDefaultData();
        }
        // Store the layer in a global variable, so the environment
        // can be completely re-built during tests using the
        // TestEnv::restoreEnv() method
        $GLOBALS['_MAX']['TEST']['layerEnv'] = $layer;
    }

    /**
     * A method to tear down the environment based on
     * the layer the test/s is/are in.
     *
     * @param string $layer The layer the test/s is/are in.
     */
    function teardownEnv($layer)
    {
        $type = $GLOBALS['_MAX']['TEST']['test_type'];
        $envType = $GLOBALS['_MAX']['TEST'][$type . '_layers'][$layer][1];
        if ($envType != NO_DB) {
            TestEnv::teardownDB();
        }
    }

}

?>
