<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: lg.php 5805 2006-10-27 10:56:56Z andrzej.swedrzynski@m3.net $
*/

/**
 * @package    MaxDelivery
 * @author     Scott Switzer <scott@switzer.org>
 */

// Require the initialisation file
require_once '../../init-delivery.php';

// Required files
require_once MAX_PATH . '/lib/max/Delivery/querystring.php';

// Prevent beacon from being cached by browsers
MAX_commonSetNoCacheHeaders();

// Remove any special characters
MAX_commonRemoveSpecialChars($_REQUEST);


/**
 * Returns a request variable exploded to an array or empty array
 * if there is no request variable with the specified name.
 * @param string Name of the variable.
 * @return array Request variable exploded to an array
 */
function _getArrRequestVariable($name)
{
	return isset($_REQUEST[$name]) ? explode(MAX_DELIVERY_MULTIPLE_DELIMITER, $_REQUEST[$name]) : array();
}


/**
 * Checks the request string for variables indicating named after a cookie.
 * If the value for the cookie is given in the request string then it returns
 * the value as an array. Otherwise, returns an empty array.
 * @param string Name of the variable in the configuration file.
 * @return array Value of the cookie in the form of array.
 */
function _getArrRequestConfVariable($name)
{
	return _getArrRequestVariable($GLOBALS['_MAX']['CONF']['var'][$name]);
}


/**
 * Returns an integer value from an array or 0 if
 * the array is empty.
 * @param array $arr
 * @param object $idx
 * @return int
 */
function _getIValueFromArr($arr, $idx)
{
	return empty($arr) ? 0 : intval($arr[$idx]);
}


/**
 * Returns an integer value from double-dimensional array or 0 if
 * the value is empty.
 * @param array $arr
 * @param object $idxFirst
 * @param object $idxSecond
 * @return int
 */
function _getIValueFromArrArr($arr, $idxFirst, $idxSecond)
{
	return _getIValueFromArr($arr[$idxFirst], $idxSecond);
}


/**
 * Reads limitation values from the array and set a proper cookies
 * according to values found.
 *
 * @param string $capType Either 'Ad' or 'Zone'.
 * @param int $id Id of the element to set limitations for.
 * @param array $arrArrCapValues 2D array to get values from.
 * @param int $idx Second index of the array to look for values.
 * @see MAX_cookieSetCapping
 */
function _setLimitations($capType, $id, $arrArrCapValues, $idx)
{
    $block = _getIValueFromArrArr($arrArrCapValues, 'block', $idx);
    $capping = _getIValueFromArrArr($arrArrCapValues, 'capping', $idx);
    $sessionCapping = _getIValueFromArrArr($arrArrCapValues, 'session_capping', $idx);
    
    MAX_cookieSetCapping($id, $capType, $block, $capping, $sessionCapping);
}


/**
 * Reads limitation values from the array and set a proper cookies
 * according to values found.
 * 
 * @param int $adId
 * @param array $arrArrCapValues
 * @param int $idx
 * @see _setLimitations
 */
function _setAdLimitations($adId, $arrArrCapValues, $idx)
{
    _setLimitations('Ad', $adId, $arrArrCapValues, $idx);
}


/**
 * Reads limitation values from the array and set a proper cookies
 * according to values found.
 * 
 * @param int $zoneId
 * @param array $arrArrCapValues
 * @param int $idx
 * @see _setLimitations
 */
function _setZoneLimitations($zoneId, $arrArrCapValues, $idx)
{
    _setLimitations('Zone', $zoneId, $arrArrCapValues, $idx);
}

$conf = $GLOBALS['_MAX']['CONF'];

// Get the variables
$viewerId   = MAX_cookieGetUniqueViewerID();
$adId       = _getArrRequestConfVariable('adId');
$zoneId     = _getArrRequestConfVariable('zoneId');
$creativeId = _getArrRequestConfVariable('creativeId');

// Get any capping information from the request array
$arrAdCapping['block']           = _getArrRequestVariable('block');
$arrAdCapping['capping']         = _getArrRequestVariable('capping');
$arrAdCapping['session_capping'] = _getArrRequestVariable('session_capping');

$arrZoneCapping['block'] = _getArrRequestConfVariable('blockZone');
$arrZoneCapping['capping'] = _getArrRequestConfVariable('capZone');
$arrZoneCapping['session_capping'] = _getArrRequestConfVariable('sessionCapZone');

// FIXME-Andrzej: Refactor these capping things using proper arrays of indices!!!

for ($i=0;$i<count($adId);$i++) {
    $adId[$i] = _getIValueFromArr($adId, $i);
    $zoneId[$i] = _getIValueFromArr($zoneId, $i);
    $creativeId[$i] = _getIValueFromArr($creativeId, $i);

    if ($adId[$i] > 0) {
        if ($conf['logging']['adImpressions']) {
            MAX_logAdImpression($viewerId, $adId[$i], $creativeId[$i], $zoneId[$i]);
        }

        _setAdLimitations($adId[$i], $arrAdCapping, $i);

        if ($zoneId[$i] != 0) {
            _setZoneLimitations($zoneId[$i], $arrZoneCapping, $i);
        }
    }
}

MAX_cookieFlush();
MAX_querystringConvertParams();

if (!empty($_REQUEST[$conf['var']['dest']])) {
    Header("Location: {$_REQUEST[$conf['var']['dest']]}");
} else {
    // Display a 1x1 pixel gif
    MAX_commonDisplay1x1();
}

// Stop benchmarking
MAX_benchmarkStop();

?>
